local _M = {}

local lfs = require "lfs"
local presetStoDir = "/data/presets"
local sysPresetStoDir = "/etc/sys-preset"
local tmd = require "kiloview.systemd-timedate"

--------------------
local function OpenPresetDB()
	local attrs = lfs.attributes( presetStoDir )
	if not attrs or attrs.mode ~= "directory" then
		return nil, nil
	end

	local userPresetId = nil

	attrs = lfs.attributes( sysPresetStoDir .. "/user" )
	if attrs then
		local f = io.popen( "/usr/bin/basename `/usr/bin/readlink " .. sysPresetStoDir .. "/user`", "r" )
		if f then
			userPresetId = f:read("l")
			f:close()
		end
	end

	local presetTable = {}
	local counter = 0

	local item
	for item in lfs.dir( presetStoDir ) do
		if item ~= "." and item ~= ".." then
			attrs = lfs.attributes( presetStoDir .. "/" .. item )
			if attrs and attrs.mode == "directory" then
				local cfgFile = presetStoDir .. "/" .. item .. "/cfg.lua"
				local x, ret = pcall( dofile, cfgFile )
				if x and type(ret) == "table" then
					counter=counter + 1
					ret.id = item
					if item == userPresetId then
						ret.is_systemDefault = true
					else
						ret.is_systemDefault = false
					end
					table.insert( presetTable, ret )
				end
			end
		end
	end

	return counter, presetTable
end

--------------------

function _M.GetPresetList()
	local counter, presetTable = OpenPresetDB()
	if not counter or not presetTable then
		return nil
	else
		return presetTable
	end
end

local config_map_table = {
	["avio.conf"] = { 
		busName = "com.kiloview.MOONLIGHT.AVIO",
		busObject = "/com/kiloview/MOONLIGHT/AVIO"
	},

	["encoder.conf"] = {
		busName = "com.kiloview.MOONLIGHT.Codec",
		busObject = "/com/kiloview/MOONLIGHT/Codec/Encoders"
	},

	["streamer.conf"] = {
	},

	["osd.conf"] = {
	},

	["audioEngine.conf"] = {
		busName = "com.kiloview.SUNSHINE.AudioEngine",
		busObject = "/com/kiloview/SUNSHINE/AudioEngine"
	},

	["hostapm.conf"] = {
	},

	["onvif.conf"] = {
	}
}

local coaf_config_map_table = {
	["avio.conf"] = { 
		busName = "codec",
		busObject = "/AVIO"
	},

	["encoder.conf"] = {
		busName = "codec",
		busObject = "/Codec/Encoders"
	},

	["streamer.conf"] = {
		busName = "streamer",
		busObject = "/DefaultApp"
	},

	["osd.conf"] = {
		busName = "codec",
		busObject = "/OSD"
	},

	["audioEngine.conf"] = {
		busName = "streamer",
		busObject = "/AudioEngine"
	},

	["hostapm.conf"] = {
		busName = "nm_daemon",
		busObject = "/raw/HOSTAP"
	},

	["onvif.conf"] = {
		busName = "onvif",
		busObject = "/",
	},

	["ptz.conf"] = {
		busName = "onvif",
		busObject = "/PTZ"
	},

	["timer.conf"] = {
		busName = "web_daemon",
		busObject = "/timer"
	},

	["web.conf"] = {
		busName = "web_daemon",
		busObject = "/SERVER"
	},

	["eth0"] = {
		busName = "nm_daemon",
		busObject = "/raw/NETWORK_MANAGER",
	},

	["ethernet-eth0"] = {
		busName = "nm_daemon",
		busObject = "/raw/NETWORK_MANAGER",
	},
}

--------------------
--
--TODO: except encoder,avio,audio, Let: (0) streamer, (1) osd (2) hostapm + network-if (3) onvif (4) web:port,users:DON't DON't!! (6) timezone
function _M.GetPresetDetail( id )
	local counter, presetTable = OpenPresetDB()
	if not counter or not presetTable then
		return nil
	else
		local i, result
		for i=1, #presetTable do
			if presetTable[i].id == id then
				result = presetTable[i]
				break
			end
		end
		
		if type(result) == "table" then
			
			result.configs = {}

			local attrs = lfs.attributes( presetStoDir .. "/" .. (result.id or "") .. "/configs" )
			if attrs and attrs.mode == "directory" then
				local item
				for item in lfs.dir( presetStoDir .. "/" .. result.id .. "/configs" ) do
					if item ~= "." and item ~= ".." then
						if _M.using_dbus then
							result.configs[ item ] = config_map_table[ item ] or {} --Or may be nil
						else
							result.configs[ item ] = coaf_config_map_table[ item ] or {} --Or may be nil
						end
						result.configs[ item ].file = presetStoDir .. "/" .. result.id .. "/configs/" .. item
						if item == "timer.conf" then
							result.configs[ item ].ltime = presetStoDir .. "/" .. result.id .. "/etc/localtime"
							result.configs[ item ].sysconf = presetStoDir .. "/" .. result.id .. "/etc/systemd/timesyncd.conf"
						end
					end
				end
			end
			
			return result
		else
			return nil
		end	
	end
end

--------------------

local function GetTelnetStatus()
	local x = io.popen( "/usr/bin/systemctl status telnet.socket", "r" )
	if not x then
		return nil
	else
		local status = "false"
		local ln = x:read("*l")
		while ln do
			local flag = ln:match( "^%s*Active:%s*(%w*)%s*.*" )
			if flag then
				if flag == "active" then
					status = "true"
				else
					status = "false"
				end
				break
			end
			ln = x:read("*l")
		end
		x:close()
		return status
	end
end

local function checkFileExist(path)
	local f = io.open(path, "r")
	local result = false
	if f then
		result = true
		f:close()
	end
	return result
end

function _M.AddPreset( name, args, saveSysSettings, saveNetSettings)
	local id
	local f = io.popen( "/usr/bin/uuidgen", "r" )
	if f then
		id = f:read("l")
		f:close()
	end
	if not id then
		print( "ERROR: Fail to generate preset unique id." )
		return nil
	end

	lfs.mkdir( presetStoDir )
	lfs.mkdir( presetStoDir .. "/" .. id )
	local cfgFile = io.open( presetStoDir .. "/" .. id .. "/cfg.lua", "w" )
	if not cfgFile then
		print( "ERROR: Fail to create the preset configure file!\n" )
		lfs.rmdir( presetStoDir .. "/" .. id )
		return nil
	end
	
	lfs.mkdir( presetStoDir .. "/" .. id .. "/configs" )

	local desc
	if type(args) == "table" then
		if args.time then
			lfs.mkdir( presetStoDir .. "/" .. id .. "/etc" )
			lfs.mkdir( presetStoDir .. "/" .. id .. "/etc/systemd" )
	
			os.execute("/bin/cp -d /etc/localtime " .. presetStoDir .. "/" .. id .. "/etc/")
			if checkFileExist("/data/configs/timer.conf") then
				os.execute("/bin/cp /data/configs/timer.conf " .. presetStoDir .. "/" .. id .. "/configs/")		
			end
			if checkFileExist("/etc/systemd/timesyncd.conf") then
				os.execute("/bin/cp /etc/systemd/timesyncd.conf " .. presetStoDir .. "/" .. id .. "/etc/systemd/" )
			end
		end
		if args.adjust and checkFileExist("/data/configs/avio.conf") then
			os.execute("/bin/cp /data/configs/avio.conf " .. presetStoDir .. "/" .. id .. "/configs/")
		end
		if args.audio and checkFileExist("/data/configs/audioEngine.conf") then
			os.execute("/bin/cp /data/configs/audioEngine.conf " .. presetStoDir .. "/" .. id .. "/configs/")
		end
		if args.code then
			if checkFileExist("/data/configs/encoder.conf") then
				os.execute("/bin/cp /data/configs/encoder.conf " .. presetStoDir .. "/" .. id .. "/configs/")
			end
			if checkFileExist("/data/configs/streamer.conf") then
				os.execute("/bin/cp /data/configs/streamer.conf " .. presetStoDir .. "/" .. id .. "/configs/")
			end
		end
		if args.overlay and checkFileExist("/data/configs/osd.conf") then
			os.execute("/bin/cp /data/configs/osd.conf " .. presetStoDir .. "/" .. id .. "/configs/")
		end
		if args.web and checkFileExist("/data/configs/web.conf") then
			os.execute("/bin/cp /data/configs/web.conf " .. presetStoDir .. "/" .. id .. "/configs/")
		end
		if args.onvif and checkFileExist("/data/configs/onvif.conf") then
			os.execute("/bin/cp /data/configs/onvif.conf " .. presetStoDir .. "/" .. id .. "/configs/")
		end
		if args.network then
			if checkFileExist("/etc/NetworkManager/system-connections/eth0") then
				os.execute("/bin/cp /etc/NetworkManager/system-connections/eth0 " .. presetStoDir .. "/" .. id .. "/configs/")
			end
			os.execute("/bin/cp /etc/NetworkManager/system-connections/ethernet-eth0 " .. presetStoDir .. "/" .. id .. "/configs/")
		end
		if args.telnet then
			local status = GetTelnetStatus()
			local f = io.open("/tmp/temp.lua", "w")
			if f then
				if status == "true" then
					f:write(
						[[
							os.execute( "/bin/ln -sf /usr/lib/systemd/system/telnet.socket /usr/lib/systemd/system/sockets.target.wants/telnet.socket" ) 
							os.execute( "/usr/bin/systemctl start telnet.socket" )
						]]
					)
				else
					f:write(
						[[
							os.execute( "/bin/rm -f /usr/lib/systemd/system/sockets.target.wants/telnet.socket" ) 
							os.execute( "/usr/bin/systemctl stop telnet.socket" )
						]]
					)
				end
				f:close()
				os.execute("/bin/mv /tmp/temp.lua " .. presetStoDir .. "/" .. id .. "/")
			end
		end
		if args.ptz and checkFileExist("/data/configs/ptz.conf") then
			os.execute("/bin/cp /data/configs/ptz.conf " .. presetStoDir .. "/" .. id .. "/configs/")
		end
	elseif type(args) == "string" then
		desc = args
		os.execute( "/bin/cp -Rf /data/configs/*.conf " .. presetStoDir .. "/" .. id .. "/configs/" )
		os.execute( "/bin/rm -f " .. presetStoDir .. "/" .. id .. "/configs/web.conf" ) --Don't save web configs.
	end
	
	if saveSysSettings then
		lfs.mkdir( presetStoDir .. "/" .. id .. "/etc" )
		lfs.mkdir( presetStoDir .. "/" .. id .. "/etc/systemd" )

		os.execute( "/bin/cp -Rf /etc/systemd/timesyncd.conf " .. presetStoDir .. "/" .. id .. "/etc/systemd/" )
	end

	if saveNetSettings then
		lfs.mkdir( presetStoDir .. "/" .. id .. "/etc/NetworkManager" )
		lfs.mkdir( presetStoDir .. "/" .. id .. "/etc/NetworkManager/system-connections" )
		os.execute( "/bin/cp -Rf /etc/NetworkManager/system-connections/* " .. presetStoDir .. "/" .. id .. "/etc/NetworkManager/system-connections/" )
	end

	cfgFile:write( [==[
return {
]==] )
	
	cfgFile:write( 'name = [==[' .. (name or id) .. ']==],\n' )
	cfgFile:write( 'desc = [==[' .. (desc or "") .. ']==],\n' )

	if saveNetSettings then
		cfgFile:write( 'netconfig = true,\n' )
	end

	if saveSysSettings then
		local tmInfo = tmd.GetTimeDateInfo()
		cfgFile:write( 'system = true,\n' )
		cfgFile:write( 'system_timedate = {\n' )
		cfgFile:write( '    zone="' .. ( tmInfo["zone"] or "UTC" ) .. '",\n' )
		cfgFile:write( '    ntp_enabled="' .. ( tmInfo["ntp_enabled"] or "no" ) .. '",\n' )
		cfgFile:write( '}\n' )
	end


	cfgFile:write( [==[
}
]==] )
	
	cfgFile:close()
	
	return id
end

--------------------------------------
function _M.PresetAsRestoreDefault( id )
	local attrs = lfs.attributes( presetStoDir .. "/" .. id )
	if attrs and attrs.mode == "directory" then
		lfs.mkdir( sysPresetStoDir )
		os.execute( "/bin/ln -sf " .. presetStoDir .. "/" .. id .. " " .. sysPresetStoDir .. "/user" )
		return true
	else
		return false, "Invalid preset"
	end
end
--------------------------------------

function _M.DeletePreset( id )
	local attrs = lfs.attributes( presetStoDir .. "/" .. id )
	if attrs and attrs.mode == "directory" then
		local userPresetId = nil
		attrs = lfs.attributes( sysPresetStoDir .. "/user" )
		if attrs then
			local f = io.popen( "/usr/bin/basename `/usr/bin/readlink " .. sysPresetStoDir .. "/user`", "r" )
			if f then
				userPresetId = f:read("l")
				f:close()
			end
		end

		if userPresetId == id then
			os.execute( "/bin/rm -Rf " .. sysPresetStoDir .. "/user" )
		end
		os.execute( "/bin/rm -Rf " .. presetStoDir .. "/" .. id )
		return true
	else
		return false, "Invalid preset"
	end
end

function _M.ExportPreset( id )
	local counter, presetTable = OpenPresetDB()
	if not counter or not presetTable then
		return nil
	end

	local find
	for _, v in pairs(presetTable) do
		if v.id == id then
			find = true
			break
		end
	end

	if not find then
		return false, "not exists this preset"
	end

	local is_ok = os.execute("/bin/mkdir -p /tmp/preset/" .. id)

	os.execute("/bin/cp -Rf " .. presetStoDir .."/" .. id .. " /tmp/preset/" .. id .. "/")
	local cpOsd = io.open(presetStoDir .."/" .. id .. "/configs/osd.conf")
	if cpOsd then
		local f = io.open("/data/osd_images", "r")
		if f then
			for item in lfs.dir( "/data/osd_images" ) do
				if item ~= "." and item ~= ".." then
					os.execute(string.format("/bin/cp -Rf /data/osd_images/%s /tmp/preset/%s/", item, id))
				end
			end
			f:close()
		end
		cpOsd:close()
	end

	local _ = os.execute("/usr/bin/zip -r -y /tmp/preset/" .. id .. ".zip   /tmp/preset/" .. id .. "/*")
	os.execute("/bin/rm -r /tmp/preset/" .. id)
	return true, "/preset/" .. id .. ".zip"
end

function _M.ImportPreset( filepath )
	local f = io.open(filepath, "r")
	if not f then
		return  false, "not find the file"
	end

	local _, presetTable = OpenPresetDB()
	local ids = {}
	for _, v in pairs(presetTable) do
		ids[v.id] = true
	end

	local tmppath = "/tmp/upload/"

	local is_ok = os.execute("/bin/mkdir -p " .. tmppath)
	if not is_ok then
		return false, "mkdir failed"
	end

	is_ok = os.execute("/usr/bin/unzip " .. filepath .. " -d " .. tmppath)
	if not is_ok then
		os.execute("/bin/rm " .. filepath)
		os.execute("/bin/rm -Rf " .. tmppath)
		return false, "tar failed"
	end

	tmppath = tmppath .. "tmp/preset"

	local find, id
	while true do
		find = nil
		for item in lfs.dir( tmppath ) do
			if item ~= "." and item ~= ".." then
				id = item
				if ids[item] then
					find = true

					local f = io.popen( "/usr/bin/uuidgen", "r" )
					if f then
						id = f:read("l")
						f:close()
					end
					if not id then
						return false, "Fail to generate preset unique id"
					end

					os.execute(string.format("/bin/mv %s/%s %s/%s", tmppath, item, tmppath, id))
					os.execute(string.format("/bin/mv %s/%s/%s %s/%s/%s", tmppath, id, item, tmppath, id, id))
					break
				end

			end
		end
		if not find then
			break
		end
	end

	os.execute(string.format("/bin/mv %s/%s/%s %s/", tmppath, id, id, presetStoDir))
	local is_ok = os.execute("/bin/mkdir -p /data/osd_image")

	for item in lfs.dir( tmppath .. "/" .. id ) do
		if item ~= "." and item ~= ".." then
			os.execute(string.format("/bin/mv /%s/%s/%s /data/osd_images/%s", tmppath, id, item, item))
		end
	end

	os.execute("/bin/rm -Rf /tmp/upload")
	return true, id
end

return _M
